indexing
	description: "Multiple precision floating-point numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPF_REF

inherit
	MP_NUMBER

	MPF_STRUCT
		export
			{NONE} all
			{MPFR_REF, MPF_REF, MPF_MATH, MPF_RANDOM, MPQ_REF, MPZ_REF} item
		redefine
			copy,
			is_equal,
			out
		end

create
	default_create,
	make_from_integer_32,
	make_from_mpfr_ref,
	make_from_mpq_ref,
	make_from_mpz_ref,
	make_from_natural_32,
	make_from_real_64,
	make_from_string,
	make_precision

convert
	make_from_integer_32 ({INTEGER_32}),
	make_from_mpfr_ref ({MPFR_REF}),
	make_from_mpz_ref ({MPZ_REF}),
	make_from_natural_32 ({NATURAL_32}),
	make_from_real_64 ({REAL_64}),
	make_from_string ({STRING_8}),
	to_real_64: {REAL_64}

feature {NONE} -- Initialization

	make_from_integer_32 (v: INTEGER_32) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpf_set_si (item, v)
		end

	make_from_mpfr_ref (v: MPFR_REF) is
			-- Initialize `Current'.
		do
			make
			mpf_init2 (item, v.precision)
			mpf_set (item, v.to_mpf_ref.item)
		end

	make_from_mpq_ref (v: MPQ_REF) is
			-- Initialize `Current' from `v'.
		do
			make_precision (v.precision)
			mpf_set_q (item, v.item)
		end

	make_from_mpz_ref (v: MPZ_REF) is
			-- Initialize `Current' from `v'.
		do
			make_precision (v.precision)
			mpf_set_z (item, v.item)
		end

	make_from_natural_32 (v: NATURAL_32) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpf_set_ui (item, v)
		end

	make_from_real_64 (v: REAL_64) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpf_set_d (item, v)
		end

	make_from_string (v: STRING_8) is
			-- Initialize `Current' from `v'.
		require
			v_not_void: v /= Void
		local
			tmp: INTEGER_32
		do
			make_precision ((v.count * 8).to_natural_32)
			tmp := mpf_set_str (item, (create {C_STRING}.make (v)).item, {BASE_CONSTANTS}.Decimal_base)
		end

	make_precision (prec: NATURAL_32) is
			-- Initialize `Current' with `precision'
			-- set to at least `prec'.
		do
			make_no_init
			mpf_init2 (item, prec)
		end

feature -- Access

	one: like Current is
			-- Neutral element for "*" and "/"
		do
			create Result.make_from_real_64 (1.0)
		end

	sign: INTEGER is
			-- Sign value (0, -1 or 1)
		do
			Result := mpf_sgn (item)
		ensure
			three_way: Result = three_way_comparison (zero)
		end

	zero: like Current is
			-- Neutral element for "+" and "-"
		do
			create Result
		end

feature -- Measurement

	precision: NATURAL_32 is
			-- Number of bits allocated for `Current'.
		do
			Result := mpf_get_prec (item)
		end

feature -- Status report

	divisible (other: like Current): BOOLEAN is
			-- May current object be divided by `other'?
		do
			Result := not other.is_equal (zero)
		end

	exponentiable (other: INTEGER_32_REF): BOOLEAN is
			-- May current object be elevated to the power `other'?
		do
			Result := (other.item >= 0) or else (not is_equal (zero))
		end

	is_integer: BOOLEAN is
			-- Is `Current' an integer (no fractional part)?
		do
			Result := mpf_integer_p (item) /= 0
		end

	fits_integer_16: BOOLEAN is
			-- Does `Current' fit in a INTEGER_16?
		do
			Result := mpf_fits_sshort_p (item) /= 0
		end

	fits_integer_32: BOOLEAN is
			-- Does `Current' fit in a INTEGER_32?
		do
			Result := mpf_fits_slong_p (item) /= 0
		end

	fits_natural_16: BOOLEAN is
			-- Does `Current' fit in a NATURAL_16?
		do
			Result := mpf_fits_ushort_p (item) /= 0
		end

	fits_natural_32: BOOLEAN is
			-- Does `Current' fit in a NATURAL_32?
		do
			Result := mpf_fits_ulong_p (item) /= 0
		end

feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is current object less than `other'?
		do
			Result := mpf_cmp (item, other.item) < 0
		end

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		do
			Result := mpf_cmp (item, other.item) = 0
		end

feature -- Element change

	set_from_integer_32 (v: INTEGER_32) is
			-- Set `Current' from `v'.
		do
			mpf_set_si (item, v)
		end

	set_from_mpq_ref (v: MPQ_REF) is
			-- Set `Current' from `v'.
		do
			mpf_set_prec (item, v.precision)
			mpf_set_q (item, v.item)
		end

	set_from_mpz_ref (v: MPZ_REF) is
			-- Set `Current' from `v'.
		do
			mpf_set_prec (item, v.precision)
			mpf_set_z (item, v.item)
		end

	set_from_natural_32 (v: NATURAL_32) is
			-- Set `Current' from `v'.
		do
			mpf_set_ui (item, v)
		end

	set_from_real_64 (v: REAL_64) is
			-- Set `Current' from `v'.
		do
			mpf_set_d (item, v)
		end

	set_from_string (v: STRING_8) is
			-- Set `Current' from `v'.
		require
			v_not_void: v /= Void
		local
			tmp: INTEGER_32
		do
			set_precision ((v.count * 8).to_natural_32)
			tmp := mpf_set_str (item, (create {C_STRING}.make (v)).item, {BASE_CONSTANTS}.Decimal_base)
		end

	set_precision (prec: NATURAL_32) is
			-- Set `precision' to at least `prec'
		do
			mpf_set_prec (item, prec)
		end

feature -- Conversion

	ceiling: MPZ_REF is
			-- Smallest integral value no smaller than current object
		local
			tmp: like Current
		do
			create tmp.make_precision (precision)
			mpf_ceil (tmp.item, item)
			create Result.make_from_mpf_ref (tmp)
		ensure
--			result_no_smaller: Result >= item
--			close_enough: Result - item < item.one
		end

	floor: MPZ_REF is
			-- Greatest integral value no greater than current object
		local
			tmp: like Current
		do
			create tmp.make_precision (precision)
			mpf_floor (tmp.item, item)
			create Result.make_from_mpf_ref (tmp)
		ensure
--			result_no_greater: Result <= item
--			close_enough: item - Result < Result.one
		end

	to_integer_32: INTEGER_32 is
			-- Convert to an INTEGER_32.
		require
			fits_integer_32: fits_integer_32
		do
			Result := mpf_get_si (item)
		end

	to_mpq_ref: MPQ_REF is
			-- Convert to an MPQ_REF.
		do
			create Result.make_from_mpf_ref (Current)
		end

	to_mpz_ref: MPZ_REF is
			-- Convert to an MPZ_REF.
		do
			create Result.make_from_mpf_ref (Current)
		end

	to_natural_32: NATURAL_32 is
			-- Convert to an NATURAL_32.
		require
			fits_natural_32: fits_natural_32
		do
			Result := mpf_get_ui (item)
		end

	to_real_64: REAL_64 is
			-- Convert to a REAL_64.
		do
			Result := mpf_get_d (item)
		end

	truncated: MPZ_REF is
			-- Integral value that represents `Current'
			-- with the fractional part truncated.
		local
			tmp: like Current
		do
			create tmp.make_precision (precision)
			mpf_trunc (tmp.item, item)
			create Result.make_from_mpf_ref (tmp)
		ensure
--			result_no_smaller: Result >= item
--			close_enough: Result - item < item.one
		end

feature -- Duplication

	copy (other: like Current) is
			-- Update current object using fields of object attached
			-- to `other', so as to yield equal objects.
		do
			if managed_pointer = Void then
				default_create
			end
			mpf_set_prec (item, other.precision)
			mpf_set (item, other.item)
		end

feature -- Basic operations

	abs: like Current is
			-- Absolute value
		do
			create Result.make_precision (precision)
			mpf_abs (Result.item, item)
		end

	infix "+" (other: like Current): like Current is
			-- Sum with `other' (commutative).
		do
			create Result.make_precision (precision.max (other.precision))
			mpf_add (Result.item, item, other.item)
		end

	infix "-" (other: like Current): like Current is
			-- Result of subtracting `other'
		do
			create Result.make_precision (precision.max (other.precision))
			mpf_sub (Result.item, item, other.item)
		end

	infix "*" (other: like Current): like Current is
			-- Product by `other'
		do
			create Result.make_precision (precision.max (other.precision))
			mpf_mul (Result.item, item, other.item)
		end

	infix "/" (other: like Current): like Current is
			-- Division by `other'
		do
			create Result.make_precision (precision.max (other.precision))
			mpf_div (Result.item, item, other.item)
		end

	infix "^" (other: INTEGER_32): like Current is
			-- Power of Current by `other'
		require
			good_exponent: exponentiable (other)
		do
			create Result.make_precision (precision)
			if other >= 0 then
				mpf_pow_ui (Result.item, item, other.to_natural_32)
			else
				mpf_pow_ui (Result.item, item, other.abs.to_natural_32)
				Result := one / Result
			end
		ensure
			result_not_void: Result /= Void
		end

	prefix "+": like Current is
			-- Unary plus
		do
			create Result.make_precision (precision)
			mpf_set (Result.item, item)
		end

	prefix "-": like Current is
			-- Unary minus
		do
			create Result.make_precision (precision)
			mpf_neg (Result.item, item)
		end

feature -- Output

	out: STRING is
			-- New string containing terse printable representation
			-- of current object
		local
			exp, i: INTEGER_32
			res: C_STRING
			str: STRING
			tmp1, tmp2: POINTER
		do
			tmp1 := mpf_get_str (tmp2, $exp, {BASE_CONSTANTS}.Decimal_base, 0, item)
			create res.make_by_pointer (tmp1)
			if is_equal (zero) then
				Result := "0"
			elseif is_integer then
				Result := res.string
			elseif exp = 0 then
				Result := "0." + res.string
			elseif exp < 0 then
				from
					i := -1
					str := "0."
				until
					i < exp
				loop
					str.append_character ('0')
					i := i - 1
				end
				Result := str + res.string
				Result.prune ('-')
				if Current < zero then
					Result.prepend_character ('-')
				end
			else
				if Current > zero then
					Result := res.string
					Result.insert_character ('.', exp + 1)
				else
					Result := res.string
					Result.insert_character ('.', exp + 2)
				end
			end
		end

invariant
	invariant_clause: True -- Your invariant here

end
