indexing
	description: "Multiple precision rational numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPQ_REF

inherit
	MP_NUMBER

	MPQ_STRUCT
		export
			{NONE} all
			{MPFR_REF, MPF_REF, MPQ_MATH, MPQ_REF, MPZ_REF} item
		redefine
			copy,
			is_equal,
			out
		end

	MPZ_FUNCTIONS
		export
			{NONE} all
		undefine
			default_create
		redefine
			copy,
			is_equal,
			out
		end

create
	default_create,
	make_from_integer,
	make_from_integer_32,
	make_from_mpfr_ref,
	make_from_mpf_ref,
	make_from_mpz_ref,
	make_from_natural_32,
	make_from_real_64,
	make_from_string

convert
	as_real_64: {REAL_64},
	make_from_integer ({INTEGER_32}),
	make_from_mpfr_ref ({MPFR_REF}),
	make_from_mpf_ref ({MPF_REF}),
	make_from_mpz_ref ({MPZ_REF}),
	make_from_real_64 ({REAL_64}),
	make_from_string ({STRING_8})

feature {NONE} -- Initialization

	make_from_integer (num: INTEGER_32) is
			-- Initialize `Current'.
		do
			default_create
			mpq_set_si (item, num, 1)
		end

	make_from_integer_32 (num: INTEGER_32; den: NATURAL_32) is
			-- Initialize `Current'.
		require
			den_not_zero: den /= 0
		do
			default_create
			mpq_set_si (item, num, den)
			mpq_canonicalize (item)
		end

	make_from_mpfr_ref (v: MPFR_REF) is
			-- Initialize `Current'.
		do
			default_create
			mpq_set_f (item, v.to_mpf_ref.item)
		end

	make_from_mpf_ref (v: MPF_REF) is
			-- Initialize `Current'.
		do
			default_create
			mpq_set_f (item, v.item)
		end

	make_from_mpz_ref (v: MPZ_REF) is
			-- Initialize `Current'.
		do
			default_create
			mpq_set_z (item, v.item)
		end

	make_from_natural_32 (num, den: NATURAL_32) is
			-- Initialize `Current'.
		require
			den_not_zero: den /= 0
		do
			default_create
			mpq_set_ui (item, num, den)
			mpq_canonicalize (item)
		end

	make_from_real_64 (v: REAL_64) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpq_set_d (item, v)
		end

	make_from_string (v: STRING_8) is
			-- Initialize `Current' from `v'.
		require
			v_not_void: v /= Void
		local
			tmp: INTEGER_32
		do
			default_create
			tmp := mpq_set_str (item, (create {C_STRING}.make (v)).item, {BASE_CONSTANTS}.Decimal_base)
			mpq_canonicalize (item)
		end

feature -- Access

	denominator: MPZ_REF is
			-- Denominator.
		do
			create Result
			mpq_get_den (Result.item, item)
		end

	numerator: MPZ_REF is
			-- Numerator.
		do
			create Result
			mpq_get_num (Result.item, item)
		end

	one: like Current is
			-- Neutral element for "*" and "/"
		do
			create Result.make_from_integer_32 (1, 1)
		end

	sign: INTEGER is
			-- Sign value (0, -1 or 1)
		do
			Result := mpq_sgn (item)
		ensure
			three_way: Result = three_way_comparison (zero)
		end

	zero: like Current is
			-- Neutral element for "+" and "-"
		do
			create Result
		end

feature -- Measurement

	precision: NATURAL_32 is
			-- Maximum precision of `numerator' and `denominator'..
		do
			Result := numerator.precision.max (denominator.precision)
		end

feature -- Status report

	divisible (other: like Current): BOOLEAN is
			-- May current object be divided by `other'?
		do
			Result := mpq_cmp_ui (other.item, 0, 1) /= 0
		end

	exponentiable (other: INTEGER_32_REF): BOOLEAN is
			-- May current object be elevated to the power `other'?
		do
			Result := (other.item >= 0) or else (not is_equal (zero))
		end

feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is current object less than `other'?
		do
			Result := mpq_cmp (item, other.item) < 0
		end

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		do
			Result := mpq_equal (item, other.item) /= 0
		end

feature -- Element change

	set (num, den: MPZ_REF) is
			-- Set `numerator' from `num'.
			-- Set `denominator' from `den'.
		do
			mpq_set_num (item, num.item)
			mpq_set_den (item, den.item)
			mpq_canonicalize (item)
		end

	set_denominator (den: MPZ_REF) is
			-- Set `denominator' from `den'.
		do
			mpq_set_den (item, den.item)
			mpq_canonicalize (item)
		end

	set_from_integer_32 (num: INTEGER_32; den: NATURAL_32) is
			-- Set `numerator' from `num'.
			-- Set `denominator' from `den'.
		require
			den_not_zero: den /= 0
		do
			mpq_set_si (item, num, den)
			mpq_canonicalize (item)
		end

	set_from_mpz_ref (v: MPZ_REF) is
			-- Set value of `Current' from `v'.
		do
			mpq_set_z (item, v.item)
		end

	set_from_natural_32 (num, den: NATURAL_32) is
			-- Set `numerator' from `num'.
			-- Set `denominator' from `den'.
		require
			den_not_zero: den /= 0
		do
			mpq_set_ui (item, num, den)
			mpq_canonicalize (item)
		end

	set_from_real_64 (v: REAL_64) is
			-- Set value of `Current' from `v'.
		do
			mpq_set_d (item, v)
		end

	set_numerator (num: MPZ_REF) is
			-- Set `numerator' from `num'.
		do
			mpq_set_num (item, num.item)
			mpq_canonicalize (item)
		end

feature -- Conversion

	as_real_64: REAL_64 is
			-- Convert `item' into an REAL_64 value.
		do
			Result := mpq_get_d (item)
		end

	to_mpf_ref: MPF_REF is
			-- Convert to an MPF_REF.
		do
			create Result.make_from_mpq_ref (Current)
		end

	to_mpz_ref: MPZ_REF is
			-- Convert to an MPZ_REF.
		do
			create Result.make_from_mpq_ref (Current)
		end

feature -- Duplication

	copy (other: like Current) is
			-- Update current object using fields of object attached
			-- to `other', so as to yield equal objects.
		do
			if managed_pointer = Void then
				default_create
			end
			mpq_set (item, other.item)
		end

feature -- Basic operations

	abs: like Current is
			-- Absolute value
		do
			create Result
			mpq_abs (Result.item, item)
		end

	infix "+" (other: like Current): like Current is
			-- Sum with `other' (commutative).
		do
			create Result
			mpq_add (Result.item, item, other.item)
		end

	infix "-" (other: like Current): like Current is
			-- Result of subtracting `other'
		do
			create Result
			mpq_sub (Result.item, item, other.item)
		end

	infix "*" (other: like Current): like Current is
			-- Product by `other'
		do
			create Result
			mpq_mul (Result.item, item, other.item)
		end

	infix "/" (other: like Current): like Current is
			-- Division by `other'
		do
			create Result
			mpq_div (Result.item, item, other.item)
		end

	infix "^" (other: INTEGER_32): like Current is
			-- Power of Current by `other'
		require
			good_exponent: exponentiable (other)
		do
			create Result
			if other >= 0 then
				Result.set (numerator ^ other.to_natural_32, denominator ^ other.to_natural_32)
			else
				Result.set (denominator ^ other.abs.to_natural_32, numerator ^ other.abs.to_natural_32)
			end
		ensure
			result_not_void: Result /= Void
		end

	prefix "+": like Current is
			-- Unary plus
		do
			create Result
			mpq_set (Result.item, item)
		end

	prefix "-": like Current is
			-- Unary minus
		do
			create Result
			mpq_neg (Result.item, item)
		end

feature -- Output

	out: STRING is
			-- New string containing terse printable representation
			-- of current object
		local
			res: C_STRING
			tmp: POINTER
		do
			create res.make_empty ((mpz_sizeinbase (numerator.item, {BASE_CONSTANTS}.Decimal_base) +
			                       mpz_sizeinbase (denominator.item, {BASE_CONSTANTS}.Decimal_base)).to_integer_32 +
			                       3)
			tmp := mpq_get_str (res.item, {BASE_CONSTANTS}.Decimal_base, item)
			Result := res.string
		end

invariant
	denominator_not_zero: not denominator.is_equal (denominator.zero)

end
