indexing
	description: "Multiple precision integer numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPZ_REF

inherit
	MP_NUMBER
		rename
			infix "/" as infix "//"
		end

	MPZ_STRUCT
		export
			{NONE} all
			{MPFR_REF,
			 MPF_REF,
			 MPQ_REF,
			 MPZ_FACTORIAL,
			 MPZ_FIBONACCI,
			 MPZ_LUCAS,
			 MPZ_MATH,
			 MPZ_PRIMES,
			 MPZ_RANDOM,
			 MPZ_REF,
			 RANDSTATE_STRUCT} item
		redefine
			copy,
			is_equal,
			out
		end

create
	default_create,
	make_from_integer_8,
	make_from_integer_16,
	make_from_integer_32,
	make_from_integer_64,
	make_from_mpfr_ref,
	make_from_mpf_ref,
	make_from_mpq_ref,
	make_from_natural_8,
	make_from_natural_16,
	make_from_natural_32,
	make_from_natural_64,
	make_from_real_64,
	make_from_string

convert
	as_real_64: {REAL_64},
	make_from_integer_32 ({INTEGER_32}),
	make_from_mpfr_ref ({MPFR_REF}),
	make_from_mpf_ref ({MPF_REF}),
	make_from_mpq_ref ({MPQ_REF}),
	make_from_natural_32 ({NATURAL_32}),
	make_from_real_64 ({REAL_64}),
	make_from_string ({STRING_8}),
	to_integer_32: {INTEGER_32},
	to_natural_32: {NATURAL_32}

feature {NONE} -- Initialization

	make_from_integer_8 (v: INTEGER_8) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpz_set_si (item, v)
		end

	make_from_integer_16 (v: INTEGER_16) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpz_set_si (item, v)
		end

	make_from_integer_32 (v: INTEGER_32) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpz_set_si (item, v)
		end

	make_from_integer_64 (v: INTEGER_64) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpz_set_si (item, v.to_integer_32)
		end

	make_from_mpfr_ref (v: MPFR_REF) is
			-- Initialize `Current'.
		do
			default_create
			mpz_set_f (item, v.to_mpf_ref.item)
		end

	make_from_mpf_ref (v: MPF_REF) is
			-- Initialize `Current'.
		do
			default_create
			mpz_set_f (item, v.item)
		end

	make_from_mpq_ref (v: MPQ_REF) is
			-- Initialize `Current'.
		do
			default_create
			mpz_set_q (item, v.item)
		end

	make_from_natural_8 (v: NATURAL_8) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpz_set_ui (item, v)
		end

	make_from_natural_16 (v: NATURAL_16) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpz_set_ui (item, v)
		end

	make_from_natural_32 (v: NATURAL_32) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpz_set_ui (item, v)
		end

	make_from_natural_64 (v: NATURAL_64) is
			-- Initialize `Current' from `v'.
		do
			default_create
			mpz_set_ui (item, v.to_natural_32)
		end

	make_from_real_64 (v: REAL_64) is
			-- Initialize `Current' from `v'.
			-- `v' will be rounded to its nearest integer value.
		do
			default_create
			mpz_set_si (item, v.rounded)
		end

	make_from_string (v: STRING_8) is
			-- Initialize `Current' from `v'.
			-- `v' can begin with an optional sign.
			-- The rest of `v' must be a string of
			-- digits containing no spaces or other
			-- characters.
		require
			v_not_void: v /= Void
			v_is_integer: is_integer_string (v)
		local
			tmp: INTEGER_32
			str: STRING_8
		do
			default_create
			if v.item (1) = '+' then
				str := v.twin
				str.prune_all_leading ('+')
				tmp := mpz_set_str (item, (create {C_STRING}.make (str)).item, {BASE_CONSTANTS}.Decimal_base)
			else
				tmp := mpz_set_str (item, (create {C_STRING}.make (v)).item, {BASE_CONSTANTS}.Decimal_base)
			end
		end

feature -- Access

	one: like Current is
			-- Neutral element for "*" and "/"
		do
			create Result.make_from_integer_32 (1)
		end

	sign: INTEGER is
			-- Sign value (0, -1 or 1)
		do
			Result := mpz_sgn (item)
		ensure
			three_way: Result = three_way_comparison (zero)
		end

	zero: like Current is
			-- Neutral element for "+" and "-"
		do
			create Result
		end

feature -- Measurement

	precision: NATURAL_32 is
			-- Number of bits allocated for `Current'.
		do
			Result := (alloc * {GMP_CONSTANTS}.Gmp_limb_bits).to_natural_32
		end

feature -- Status report

	divisible (other: like Current): BOOLEAN is
			-- May current object be divided by `other'?
		do
			Result := mpz_cmp_ui (other.item, 0) /= 0
		end

	exponentiable (other: NATURAL_32_REF): BOOLEAN is
			-- May current object be elevated to the power `other'?
		do
			Result := True
		end

	fits_integer_16: BOOLEAN is
			-- Does `Current' fit in an INTEGER_16?
		do
			Result := mpz_fits_sshort_p (item) /= 0
		end

	fits_integer_32: BOOLEAN is
			-- Does `Current' fit in an INTEGER_32?
		do
			Result := mpz_fits_slong_p (item) /= 0
		end

	fits_natural_16: BOOLEAN is
			-- Does `Current' fit in an NATURAL_16?
		do
			Result := mpz_fits_ushort_p (item) /= 0
		end

	fits_natural_32: BOOLEAN is
			-- Does `Current' fit in an NATURAL_32?
		do
			Result := mpz_fits_ulong_p (item) /= 0
		end

	is_even: BOOLEAN is
			-- Is `Current' and even integer?
		do
			Result := mpz_even_p (item) /= 0
		end

	is_integer_string (str: STRING_8): BOOLEAN is
			-- Does `str' represent an integer?
		local
			i: INTEGER_32
			count: INTEGER_32
		do
			if str.item (1).is_digit then
				Result := True
			else
				Result := (str.item (1) = '+') or else (str.item (1) = '-')
			end
			if Result then
				from
					i := 2
					count := str.count
				until
					(not Result) or else (i > count)
				loop
					Result := str.item (i).is_digit
					i := i + 1
				end
			end
		end

	is_odd: BOOLEAN is
			-- Is `Current' and odd integer?
		do
			Result := mpz_odd_p (item) /= 0
		end

	is_perfect_power: BOOLEAN is
			-- Is `Current' a perfect power?
		do
			Result := mpz_perfect_power_p (item) /= 0
		end

	is_perfect_square: BOOLEAN is
			-- Is `Current' a perfect square?
		do
			Result := mpz_perfect_square_p (item) /= 0
		end

	is_prime: BOOLEAN is
			-- Is `Current' prime?
		do
			Result := mpz_probab_prime_p (item, 10) = 2
		end

feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is current object less than `other'?
		do
			Result := mpz_cmp (item, other.item) < 0
		end

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		do
			Result := mpz_cmp (item, other.item) = 0
		end

feature -- Element change

	set_from_integer_8 (v: INTEGER_8) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_si (item, v)
		end

	set_from_integer_16 (v: INTEGER_16) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_si (item, v)
		end

	set_from_integer_32 (v: INTEGER_32) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_si (item, v)
		end

	set_from_integer_64 (v: INTEGER_64) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_si (item, v.to_integer_32)
		end

	set_from_mpf_ref (v: MPF_REF) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_f (item, v.item)
		end

	set_from_mpq_ref (v: MPQ_REF) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_q (item, v.item)
		end

	set_from_natural_8 (v: NATURAL_8) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_ui (item, v)
		end

	set_from_natural_16 (v: NATURAL_16) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_ui (item, v)
		end

	set_from_natural_32 (v: NATURAL_32) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_ui (item, v)
		end

	set_from_natural_64 (v: NATURAL_64) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_ui (item, v.to_natural_32)
		end

	set_from_real_64 (v: REAL_64) is
			-- Set value of `Current' from `v'.
		do
			mpz_set_d (item, v)
		end

	set_from_string (v: STRING_8) is
			-- Set value of `Current' from `v'.
			-- `v' can begin with an optional sign.
			-- The rest of `v' must be a string of
			-- digits containing no spaces or other
			-- characters.
		require
			v_not_void: v /= Void
			v_is_integer: is_integer_string (v)
		local
			tmp: INTEGER_32
			str: STRING_8
		do
			if v.item (1) = '+' then
				str := v.twin
				str.prune_all_leading ('+')
				tmp := mpz_set_str (item, (create {C_STRING}.make (str)).item, {BASE_CONSTANTS}.Decimal_base)
			else
				tmp := mpz_set_str (item, (create {C_STRING}.make (v)).item, {BASE_CONSTANTS}.Decimal_base)
			end
		end

feature -- Conversion

	as_integer_32: INTEGER_32 is
			-- Convert `item' into an INTEGER_32 value.
		do
			Result := mpz_get_si (item)
		end

	as_natural_32: NATURAL_32 is
			-- Convert `item' into an NATURAL_32 value.
		do
			Result := mpz_get_ui (item)
		end

	as_real_64: REAL_64 is
			-- Convert `item' into an REAL_64 value.
		do
			Result := mpz_get_d (item)
		end

	to_bin_string: STRING is
			-- Convert to a binary string.
		local
			res: C_STRING
			tmp: POINTER
		do
			create res.make_empty (mpz_sizeinbase (item, {BASE_CONSTANTS}.Binary_base).to_integer_32 + 2)
			tmp := mpz_get_str (res.item, {BASE_CONSTANTS}.Binary_base, item)
			Result := res.string
		end

	to_hex_string: STRING is
			-- Convert to a hexadecimal string.
		local
			res: C_STRING
			tmp: POINTER
		do
			create res.make_empty (mpz_sizeinbase (item, {BASE_CONSTANTS}.Hexadecimal_base).to_integer_32 + 2)
			tmp := mpz_get_str (res.item, {BASE_CONSTANTS}.Hexadecimal_base, item)
			Result := res.string
		end

	to_integer_32: INTEGER_32 is
			-- Convert `item' into an INTEGER_32 value.
		require
			fits_integer_32: fits_integer_32
		do
			Result := mpz_get_si (item)
		end

	to_mpfr_ref: MPFR_REF is
			-- Convert to an MPFR_REF.
		do
			create Result.make_from_mpz_ref (Current)
		end

	to_mpf_ref: MPF_REF is
			-- Convert to an MPF_REF.
		do
			create Result.make_from_mpz_ref (Current)
		end

	to_mpq_ref: MPQ_REF is
			-- Convert to an MPQ_REF.
		do
			create Result.make_from_mpz_ref (Current)
		end

	to_natural_32: NATURAL_32 is
			-- Convert `item' into an NATURAL_32 value.
		require
			fits_natural_32: fits_natural_32
		do
			Result := mpz_get_ui (item)
		end

	to_oct_string: STRING is
			-- Convert to a octal string.
		local
			res: C_STRING
			tmp: POINTER
		do
			create res.make_empty (mpz_sizeinbase (item, {BASE_CONSTANTS}.Octal_base).to_integer_32 + 2)
			tmp := mpz_get_str (res.item, {BASE_CONSTANTS}.Octal_base, item)
			Result := res.string
		end

feature -- Duplication

	copy (other: like Current) is
			-- Update current object using fields of object attached
			-- to `other', so as to yield equal objects.
		do
			if managed_pointer = Void then
				default_create
			end
			mpz_set (item, other.item)
		end

feature -- Basic operations

	abs: like Current is
			-- Absolute value
		do
			create Result
			mpz_abs (Result.item, item)
		end

	infix "+" (other: like Current): like Current is
			-- Sum with `other' (commutative).
		do
			create Result
			mpz_add (Result.item, item, other.item)
		end

	infix "-" (other: like Current): like Current is
			-- Result of subtracting `other'
		do
			create Result
			mpz_sub (Result.item, item, other.item)
		end

	infix "*" (other: like Current): like Current is
			-- Product by `other'
		do
			create Result
			mpz_mul (Result.item, item, other.item)
		end

	infix "/" (other: like Current): MPF_REF is
			-- Integer division by `other'
		require
			other_not_void: other /= Void
			good_divisor: divisible (other)
		do
			Result := create {MPF_REF}.make_from_mpz_ref (Current) / create {MPF_REF}.make_from_mpz_ref (other)
		ensure
			result_not_void: Result /= Void
		end

	infix "//" (other: like Current): like Current is
			-- Integer division by `other'
		do
			create Result
			mpz_tdiv_q (Result.item, item, other.item)
		end

	infix "\\" (other: like Current): like Current is
			-- Remainder of the integer division of Current by `other'
		do
			create Result
			mpz_mod (Result.item, item, other.item)
		end

	infix "^" (other: NATURAL_32): like Current is
			-- Integer power of Current by `other'
		do
			create Result
			mpz_pow_ui (Result.item, item, other.item)
		ensure
			result_not_void: Result /= Void
		end

	prefix "+": like Current is
			-- Unary plus
		do
			create Result
			mpz_set (Result.item, item)
		end

	prefix "-": like Current is
			-- Unary minus
		do
			create Result
			mpz_neg (Result.item, item)
		end

feature -- Bit operations

	infix "&", bit_and (other: like Current): like Current is
			-- Bitwise and between Current' and `other'.
		require
			other_not_void: other /= Void
		do
			create Result
			mpz_and (Result.item, item, other.item)
		ensure
			bitwise_and_not_void: Result /= Void
		end

	infix "|", bit_or (other: like Current): like Current is
			-- Bitwise or between Current' and `other'.
		require
			other_not_void: other /= Void
		do
			create Result
			mpz_ior (Result.item, item, other.item)
		ensure
			bitwise_or_not_void: Result /= Void
		end

	bit_xor (other: like Current): like Current is
			-- Bitwise xor between Current' and `other'.
		require
			other_not_void: other /= Void
		do
			create Result
			mpz_xor (Result.item, item, other.item)
		ensure
			bitwise_xor_not_void: Result /= Void
		end

	bit_not: like Current is
			-- One's complement of Current.
		do
			create Result
			mpz_com (Result.item, item)
		ensure
			bit_not_not_void: Result /= Void
		end

	frozen bit_test (n: NATURAL_32): BOOLEAN is
			-- Test `n'-th position of Current.
		require
			n_nonnegative: n >= 0
			n_less_than_precision: n < precision
		do
			Result := mpz_tstbit (item, n).to_boolean
		end

	frozen set_bit (b: BOOLEAN; n: NATURAL_32): like Current is
			-- Copy of current with `n'-th position
			-- set to 1 if `b', 0 otherwise.
		require
			n_nonnegative: n >= 0
			n_less_than_precision: n < precision
		do
			create Result
			Result.copy (Current)
			if b then
				mpz_setbit (Result.item, n)
			else
				mpz_clrbit (Result.item, n)
			end
		end

feature -- Output

	out: STRING is
			-- New string containing terse printable representation
			-- of current object
		local
			res: C_STRING
			tmp: POINTER
		do
			create res.make_empty (mpz_sizeinbase (item, {BASE_CONSTANTS}.Decimal_base).to_integer_32 + 2)
			tmp := mpz_get_str (res.item, {BASE_CONSTANTS}.Decimal_base, item)
			Result := res.string
		end

end
