indexing
	description: "Mathematical operations on integer numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPZ_MATH

inherit
	MPZ_FUNCTIONS
		export
			{NONE} all
		end

feature -- Arithmetic

	add_mul (res, a, b: MPZ_REF) is
			-- `res' := `res' + `a' * `b'.
		require
			res_not_void: res /= Void
			a_not_void: a /= Void
			b_not_void: b /= Void
		do
			mpz_addmul (res.item, a.item, b.item)
		end

	sub_mul (res, a, b: MPZ_REF) is
			-- `res' := `res' - `a' * `b'.
		require
			res_not_void: res /= Void
			a_not_void: a /= Void
			b_not_void: b /= Void
		do
			mpz_submul (res.item, a.item, b.item)
		end

feature -- Exponentiation

	pow_mod (base, exp, mod: MPZ_REF): MPZ_REF is
			-- Set `Result' to (`base' raised to `exp') modulo `mod'.
			-- Negative `exp' is supported if an inverse `base^-1' mod
			-- `mod' exists
		require
			base_not_void: base /= Void
			exp_not_void: exp /= Void
			mod_not_void: mod /= Void
		do
			create Result
			mpz_powm (Result.item, base.item, exp.item, mod.item)
		end

feature -- Measurement

	factors_removed: INTEGER_32
			-- Numver of factors removed by `remove'.

feature -- Number theoretic

	binomial_coefficient (n: MPZ_REF; k: NATURAL_32): MPZ_REF is
			-- Binomial coefficient `n over k'.
		require
			n_not_void: n /= Void
		do
			create Result
			mpz_bin_ui (Result.item, n.item, k)
		ensure
			result_not_void: Result /= Void
		end

	gcd (z1, z2: MPZ_REF): MPZ_REF is
			-- Greatest common divizor of `z1 and `z2'.
		require
			z1_not_void: z1 /= Void
			z2_not_void: z2 /= Void
		do
			create Result
			mpz_gcd (Result.item, z1.item, z2.item)
		ensure
			result_not_void: Result /= Void
		end

	jacobi (a, b: MPZ_REF): INTEGER_32 is
			-- Jacobi symbol.
		require
			a_not_void: a /= Void
			b_not_void: b /= Void
			b_odd: b.is_odd
		do
			Result := mpz_jacobi (a.item, b.item)
		end

	kronecker (a, b: MPZ_REF): INTEGER_32 is
			-- Jacobi symbol `(a/b)' with the Kronecker
			-- extension `(a/2)=(2/a)' when `a' odd, or
			-- `(a/2)=0' when `a' even.
		require
			a_not_void: a /= Void
			b_not_void: b /= Void
			b_odd: b.is_odd
		do
			Result := mpz_jacobi (a.item, b.item)
		end

	lcm (z1, z2: MPZ_REF): MPZ_REF is
			-- Least common multiple of `z1 and `z2'.
		require
			z1_not_void: z1 /= Void
			z2_not_void: z2 /= Void
		do
			mpz_lcm (Result.item, z1.item, z2.item)
		ensure
			result_not_void: Result /= Void
		end

	legendre (a, p: MPZ_REF): INTEGER_32 is
			-- Legendre symbol.
		require
			a_not_void: a /= Void
			p_not_void: p /= Void
			p_odd: p.is_odd
			p_positive: p > 0
			p_is_prime: p.is_prime
		do
			Result := mpz_legendre (a.item, p.item)
		end

	remove (z, f: MPZ_REF): MPZ_REF is
			-- Remove all occurrences of the factor `f' from `f'
			-- and store the result in `Result'.
			-- `factors_removed' is set to how many such occurrences
			-- were removed.
		require
			z_not_void: z /= Void
			f_not_void: f /= Void
		do
			create Result
			factors_removed := mpz_remove (Result.item, z.item, f.item)
		end

feature -- Root extraction

	root (z: MPZ_REF; n: NATURAL_32): MPZ_REF is
			-- Truncated integer part of the `n'th root of `z'.
		require
			z_not_void: z /= Void
		local
			res: INTEGER_32
		do
			create Result
			res := mpz_root (Result.item, z.item, n)
		ensure
			result_not_void: Result /= Void
		end

	root_remainder (z: MPZ_REF; n: NATURAL_32): TUPLE [MPZ_REF, MPZ_REF] is
			-- Set `Result [1]' to the truncated integer part of the `n'th
			-- root of `z'.
			-- Set `rem' to the remainder `z - Result [1]^n'.
		require
			z_not_void: z /= Void
		local
			tmp1, tmp2: MPZ_REF
		do
			create Result
			create tmp1
			create tmp2
			mpz_rootrem (tmp1.item, tmp2.item, z.item, n)
			Result [1] := tmp1
			Result [2] := tmp2
		ensure
			result_not_void: Result /= Void
		end

	sqrt (z: MPZ_REF): MPZ_REF is
			-- Truncated integer part of the wquare root of `z'.
		require
			z_not_void: z /= Void
		do
			create Result
			mpz_sqrt (Result.item, z.item)
		ensure
			result_not_void: Result /= Void
		end

	sqrt_remainder (z: MPZ_REF): TUPLE [MPZ_REF, MPZ_REF] is
			-- Set `Result [1]' to the truncated integer part of the square root of `z'
			-- Set `Result [2]' to the remainder `z - Result [1]*Result [1]',
			-- which will be zero if `op' is a perfect square.
		require
			z_not_void: z /= Void
		local
			tmp1, tmp2: MPZ_REF
		do
			create Result
			create tmp1
			create tmp2
			mpz_sqrtrem (tmp1.item, tmp2.item, z.item)
			Result [1] := tmp1
			Result [2] := tmp2
		ensure
			result_not_void: Result /= Void
		end

end
