indexing
	description: "Mathematical operations on complex numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPFR_COMPLEX_MATH

inherit
	MPFR_FUNCTIONS
		export
			{NONE} all
		end

feature -- Access

	last_result: INTEGER_32
			-- Last result of a call to
			-- an MPFR function.

feature -- Exponential

	exp (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Exponential of `z'.
			-- e ^ `z'
		require
			z_not_void: z /= Void
		local
			coeff, tmp_sin, tmp_cos: MPFR_REF
		do
			create coeff.make_precision_nan (re_prec.max (im_prec), create {MPFR_RND_T}.default_create)
			create tmp_cos.make_precision_nan (re_prec, re_rnd)
			tmp_cos.set_rounding_mode (re_rnd)
			create tmp_sin.make_precision_nan (im_prec, im_rnd)
			tmp_sin.set_rounding_mode (im_rnd)
			last_result := mpfr_exp (coeff.item, z.real.item, coeff.rounding_mode)
			last_result := mpfr_cos (tmp_cos.item, z.imaginary.item, re_rnd)
			last_result := mpfr_sin (tmp_sin.item, z.imaginary.item, im_rnd)
			create Result.make (tmp_cos * coeff, tmp_sin * coeff)
		ensure
			result_not_void: Result /= Void
			result_rounding_modes_correct: (Result.real.rounding_mode = z.real.rounding_mode) and
			                               (Result.imaginary.rounding_mode = z.imaginary.rounding_mode)
		end

feature -- Hyperbolic

	cosh (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Hyperbolic cosine of `z'.
		require
			z_not_void: z /= Void
		local
			tmp_cosh, tmp_cos, tmp_sinh, tmp_sin: MPFR_REF
		do
			create tmp_cosh.make_precision_nan (re_prec, re_rnd)
			create tmp_cos.make_precision_nan (re_prec, re_rnd)
			create tmp_sinh.make_precision_nan (im_prec, im_rnd)
			create tmp_sin.make_precision_nan (im_prec, im_rnd)
			last_result := mpfr_cosh (tmp_cosh.item, z.real.item, re_rnd)
			last_result := mpfr_cos (tmp_cos.item, z.imaginary.item, re_rnd)
			last_result := mpfr_sinh (tmp_sinh.item, z.real.item, im_rnd)
			last_result := mpfr_sin (tmp_sin.item, z.imaginary.item, im_rnd)
			create Result.make (tmp_cosh * tmp_cos, tmp_sinh * tmp_sin)
		ensure
			result_not_void: Result /= Void
		end

	coth (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Hyperbolic cotangent of `z'.
		require
			z_not_void: z /= Void
		do
			Result := One / tanh (z, re_prec, im_prec, re_rnd, im_rnd)
		end

	csch (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Hyperbolic cosecant of `z'.
		require
			z_not_void: z /= Void
		do
			Result := One / sinh (z, re_prec, im_prec, re_rnd, im_rnd)
		end

	sech (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Hyperbolic secant of `z'.
		require
			z_not_void: z /= Void
		do
			Result := One / cosh (z, re_prec, im_prec, re_rnd, im_rnd)
		end

	sinh (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Hyperbolic sine of `z'.
		require
			z_not_void: z /= Void
		local
			tmp_sinh, tmp_cos, tmp_cosh, tmp_sin: MPFR_REF
		do
			create tmp_sinh.make_precision_nan (re_prec, re_rnd)
			create tmp_cos.make_precision_nan (re_prec, re_rnd)
			create tmp_cosh.make_precision_nan (im_prec, im_rnd)
			create tmp_sin.make_precision_nan (im_prec, im_rnd)
			last_result := mpfr_sinh (tmp_sinh.item, z.real.item, re_rnd)
			last_result := mpfr_cos (tmp_cos.item, z.imaginary.item, re_rnd)
			last_result := mpfr_cosh (tmp_cosh.item, z.real.item, im_rnd)
			last_result := mpfr_sin (tmp_sin.item, z.imaginary.item, im_rnd)
			create Result.make (tmp_sinh * tmp_cos, tmp_cosh * tmp_sin)
		ensure
			result_not_void: Result /= Void
		end

	tanh (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Hyperbolic tangent of `z'.
		require
			z_not_void: z /= Void
		local
			x2, y2, div, tmp_cosh, tmp_cos, tmp_sinh, tmp_sin: MPFR_REF
			prec: NATURAL_32
			def_rnd: MPFR_RND_T
		do
			x2 := z.real + z.real
			y2 := z.imaginary + z.imaginary
			prec := re_prec.max (im_prec)
			create def_rnd.default_create
			create tmp_cosh.make_precision_nan (prec, def_rnd)
			create tmp_cos.make_precision_nan (prec, def_rnd)
			last_result := mpfr_cosh (tmp_cosh.item, x2.item, def_rnd)
			last_result := mpfr_cos (tmp_cos.item, y2.item, def_rnd)
			create div.make_precision_nan (prec, def_rnd)
			div := div.one / (tmp_cosh / tmp_cos)
			create tmp_sinh.make_precision_nan (re_prec, re_rnd)
			last_result := mpfr_sinh (tmp_sinh.item, x2.item, re_rnd)
			create tmp_sin.make_precision_nan (im_prec, im_rnd)
			last_result := mpfr_sin (tmp_sin.item, y2.item, im_rnd)
			create Result.make (tmp_sinh * div, tmp_sin * div)
		ensure
			result_not_void: Result /= Void
		end

feature -- Inverse hyperbolic

	acosh (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc hyperbolic cosine of `z'.
		require
			z_not_void: z /= Void
		local
			quad: INTEGER_32
		do
			Result := log (z + principal_sqrt (z.squared - One, re_prec, im_prec, re_rnd, im_rnd),
			               re_prec, im_prec, re_rnd, im_rnd)
			quad := z.quadrant
			if ((quad = 2) or else (quad = 3)) then
				Result := -Result
			end
		end

	acoth (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc hyperbolic cotangent of `z'.
		require
			z_not_void: z /= Void
		local
			recip: MPFR_COMPLEX_REF
		do
			recip := One / z
			Result := (log (recip + One, re_prec, im_prec, re_rnd, im_rnd) -
			           log (One - recip, re_prec, im_prec, re_rnd, im_rnd)) / Two
		end

	acsch (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc hyperbolic cosecant of `z'.
		require
			z_not_void: z /= Void
		do
			Result := asinh (One / z, re_prec, im_prec, re_rnd, im_rnd)
		end

	asech (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc hyperbolic secant of `z'.
		require
			z_not_void: z /= Void
		do
			Result := acosh (One / z, re_prec, im_prec, re_rnd, im_rnd)
		end

	asinh (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc hyperbolic sine of `z'.
		require
			z_not_void: z /= Void
		do
			Result := log (z + principal_sqrt (z.squared + One, re_prec, im_prec, re_rnd, im_rnd),
			               re_prec, im_prec, re_rnd, im_rnd)
		end

	atanh (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc hyperbolic tangent of `z'.
		require
			z_not_void: z /= Void
		local
			pi: MPFR_REF
			tmp: INTEGER_32
		do
			create pi.make_precision_nan (re_prec, create {MPFR_RND_T}.default_create)
			tmp := mpfr_const_pi (pi.item, re_rnd)
			pi.set_rounding_mode (re_rnd)
			if z.imaginary >= z.imaginary.zero then
				Result := times_i (acoth (z, re_prec, im_prec, re_rnd, im_rnd) +
				                   create {MPFR_COMPLEX_REF}.make (pi, z.imaginary)) / Two
			else
				Result := times_i (acoth (z, re_prec, im_prec, re_rnd, im_rnd) -
				                   create {MPFR_COMPLEX_REF}.make (pi, z.imaginary)) / Two
			end
		end

feature -- Inverse trigonometric

	acos (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc cosine of `z'.
		require
			z_not_void: z /= Void
		do
			Result := log (z + times_i (principal_sqrt (One - z.squared, re_prec, im_prec, re_rnd, im_rnd)),
			               re_prec, im_prec, re_rnd, im_rnd)
			Result := -times_i (Result)
		end

	acot (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc cotangent of `z'.
		require
			z_not_void: z /= Void
		do
			Result := atan (One / z, re_prec, im_prec, re_rnd, im_rnd)
		end

	acsc (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc cosecant of `z'.
		require
			z_not_void: z /= Void
		do
			Result := asin (One / z, re_prec, im_prec, re_rnd, im_rnd)
		end

	asec (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc secant of `z'.
		require
			z_not_void: z /= Void
		do
			Result := acos (One / z, re_prec, im_prec, re_rnd, im_rnd)
		end

	asin (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc sine of `z'.
		require
			z_not_void: z /= Void
		do
			Result := times_i (log (times_i (z), re_prec, im_prec, re_rnd, im_rnd) + (One - z.squared))
		end

	atan (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Arc tangent of `z'.
		require
			z_not_void: z /= Void
		do
			Result := (log (One - times_i (z), re_prec, im_prec, re_rnd, im_rnd) -
			           log (One + times_i (z), re_prec, im_prec, re_rnd, im_rnd)) / Two
		end

feature -- Logarithmic

	log (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Natural logarithm of `z'
		require
			z_not_void: z /= Void
		local
			hypot, atan2: MPFR_REF
		do
			create hypot.make_precision_nan (re_prec, re_rnd)
			last_result := mpfr_hypot (hypot.item, z.real.item, z.imaginary.item, re_rnd)
			create atan2.make_precision_nan (im_prec, im_rnd)
			last_result := mpfr_atan2 (atan2.item, z.imaginary.item, z.real.item, im_rnd)
			last_result := mpfr_log (hypot.item, hypot.item, re_rnd)
			create Result.make (hypot, atan2)
		end

	log2 (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Base 2 logarithm of `z'
		require
			z_not_void: z /= Void
		do
			Result := log (z, re_prec, im_prec, re_rnd, im_rnd) / log (Two, re_prec, im_prec, re_rnd, im_rnd)
		ensure
			result_not_void: Result /= Void
		end

	log10 (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Base 10 logarithm of `z'
		require
			z_not_void: z /= Void
		do
			Result := log (z, re_prec, im_prec, re_rnd, im_rnd) / log (Ten, re_prec, im_prec, re_rnd, im_rnd)
		ensure
			result_not_void: Result /= Void
		end

	logx (z, x: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Base `x' logarithm of `z'
		require
			z_not_void: z /= Void
			x_not_void: x /= Void
		do
			Result := log (z, re_prec, im_prec, re_rnd, im_rnd) / log (x, re_prec, im_prec, re_rnd, im_rnd)
		ensure
			result_not_void: Result /= Void
		end

feature -- Root extraction

	principal_root (z: MPFR_COMPLEX_REF;
	                r_th: INTEGER_32;
	                re_prec, im_prec: NATURAL_32;
	                re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- The r-th principal root of `z'.
		require
			z_not_void: z /= Void
			r_th_positive: r_th > 0
		local
			theta, n, tmp_cos, tmp_sin: MPFR_REF
			prec: NATURAL_32
			def_rnd: MPFR_RND_T
		do
			create def_rnd.default_create
			prec := re_prec.max (im_prec)
			create theta.make_precision_nan (prec, def_rnd)
			last_result := mpfr_atan2 (theta.item, z.imaginary.item, z.real.item, def_rnd)
			create n.make_precision_nan (prec, def_rnd)
			last_result := mpfr_hypot (n.item, z.real.item, z.imaginary.item, def_rnd)
			create tmp_cos.make_precision_nan (re_prec, re_rnd)
			last_result := mpfr_cos (tmp_cos.item, theta.item, re_rnd)
			create tmp_sin.make_precision_nan (im_prec, im_rnd)
			tmp_sin.set_rounding_mode (im_rnd)
			last_result := mpfr_sin (tmp_sin.item, theta.item, im_rnd)
			create Result.make (tmp_cos * n, tmp_sin * n)
		end

	principal_sqrt (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Square root of `z'.
		require
			z_not_void: z /= Void
		local
			t, sign, re, im: MPFR_REF
		do
			re := z.real
			im := z.imaginary
			if re >= re.zero then
				create t.make_precision_nan (re_prec, re_rnd)
				last_result := mpfr_sqrt (t.item, ((re * re) + (im * im)).item, re_rnd)
				t := (t + re.abs) / Two_mpfr
				last_result := mpfr_sqrt (t.item, t.item, re_rnd)
				create im.make_precision_nan (im_prec, im_rnd)
				last_result := mpfr_set (im.item, z.imaginary.item, im_rnd)
				create Result.make (t, im / (Two_mpfr * t))
			else
				create t.make_precision_nan (re_prec.max (im_prec), im_rnd)
				last_result := mpfr_sqrt (t.item, ((re * re) + (im * im)).item, t.rounding_mode)
				t := (t + re.abs) / Two_mpfr
				last_result := mpfr_sqrt (t.item, t.item, re_rnd)
				if im < im.zero then
					sign := z.real.one
				elseif im > im.zero then
					sign := -z.real.one
				else
					sign := z.real.zero
				end
				create im.make_precision_nan (re_prec, re_rnd)
				last_result := mpfr_set (im.item, z.imaginary.item, re_rnd)
				create Result.make (im.abs / (Two_mpfr * t), t * sign)
			end
		end

	root (z: MPFR_COMPLEX_REF;
	      r_th, i_th: INTEGER_32;
	      re_prec, im_prec: NATURAL_32;
	      re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- The i-th, r-th root of `z'.
			-- There are `r' r-th roots numbered from 1 to r and
			-- `i_th' selects one.
		require
			z_not_void: z /= Void
			r_th_positive: r_th > 0
			i_th_positive: i_th > 0
			i_th_in_range: i_th <= r_th
		local
			theta, n, tmp_cos, tmp_sin: MPFR_REF
			prec: NATURAL_32
			def_rnd: MPFR_RND_T
		do
			prec := re_prec.max (im_prec)
			create def_rnd.default_create
			create theta.make_precision_nan (prec, def_rnd)
			last_result := mpfr_atan2 (theta.item, z.imaginary.item, z.real.item, theta.rounding_mode)
			theta := theta +
			         Two_mpfr *
			         create {MPFR_REF}.make_from_integer_32 (i_th - 1, theta.precision, create {MPFR_RND_T}.default_create) *
			         (create {MPFR_MATH}).pi (theta.precision, theta.rounding_mode)
			create n.make_precision_nan (prec, def_rnd)
			last_result := mpfr_hypot (n.item, z.real.item, z.imaginary.item, n.rounding_mode)
			create tmp_cos.make_precision_nan (re_prec, re_rnd)
			last_result := mpfr_cos (tmp_cos.item, theta.item, re_rnd)
			create tmp_sin.make_precision_nan (im_prec, im_rnd)
			last_result := mpfr_sin (tmp_sin.item, theta.item, z.imaginary.rounding_mode)
			create Result.make (tmp_cos * n, tmp_sin * n)
		end

	sqrt (z: MPFR_COMPLEX_REF;
	      i_th: INTEGER_32;
	      re_prec, im_prec: NATURAL_32;
	      re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Square root of `z'.
		require
			z_not_void: z /= Void
			i_th_one_or_two: (i_th = 1) or (i_th = 2)
		do
			if i_th = 1 then
				Result := principal_sqrt (z, re_prec, im_prec, re_rnd, im_rnd)
			else
				Result := principal_sqrt (z, re_prec, im_prec, re_rnd, im_rnd)
				Result.set_imaginary (-Result.imaginary)
			end
		end

feature -- Trigonometric

	cos (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Coine of `z'
		require
			z_not_void: z /= Void
		local
			tmp_cos, tmp_cosh, tmp_sin, tmp_sinh: MPFR_REF
		do
			create tmp_cos.make_precision_nan (re_prec, re_rnd)
			create tmp_cosh.make_precision_nan (re_prec, re_rnd)
			create tmp_sin.make_precision_nan (im_prec, im_rnd)
			create tmp_sinh.make_precision_nan (im_prec, im_rnd)
			last_result := mpfr_cos (tmp_cos.item, z.real.item, re_rnd)
			last_result := mpfr_cosh (tmp_cosh.item, z.imaginary.item, re_rnd)
			last_result := mpfr_sin (tmp_sin.item, z.real.item, im_rnd)
			last_result := mpfr_sinh (tmp_sinh.item, z.imaginary.item, im_rnd)
			create Result.make (tmp_cos * tmp_cosh, tmp_sin * tmp_sinh)
		end

	cot (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Cotangent of `z'
		require
			z_not_void: z /= Void
		do
			Result := tan (One / z, re_prec, im_prec, re_rnd, im_rnd)
		end

	csc (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Cosecant of `z'
		require
			z_not_void: z /= Void
		do
			Result := sin (One / z, re_prec, im_prec, re_rnd, im_rnd)
		end

	sec (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Secant of `z'
		require
			z_not_void: z /= Void
		do
			Result := cos (One / z, re_prec, im_prec, re_rnd, im_rnd)
		end

	sin (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Sine of `z'
		require
			z_not_void: z /= Void
		local
			tmp_sin, tmp_cosh, tmp_cos, tmp_sinh: MPFR_REF
		do
			create tmp_sin.make_precision_nan (re_prec, re_rnd)
			create tmp_cosh.make_precision_nan (re_prec, re_rnd)
			create tmp_cos.make_precision_nan (im_prec, im_rnd)
			create tmp_sinh.make_precision_nan (im_prec, im_rnd)
			last_result := mpfr_sin (tmp_sin.item, z.real.item, re_rnd)
			last_result := mpfr_cosh (tmp_cosh.item, z.imaginary.item, re_rnd)
			last_result := mpfr_cos (tmp_cos.item, z.real.item, im_rnd)
			last_result := mpfr_sinh (tmp_sinh.item, z.imaginary.item, im_rnd)
			create Result.make (tmp_sin * tmp_cosh, tmp_cos * tmp_sinh)
		end

	tan (z: MPFR_COMPLEX_REF; re_prec, im_prec: NATURAL_32; re_rnd, im_rnd: MPFR_RND_T): MPFR_COMPLEX_REF is
			-- Tangent of `z'
		require
			z_not_void: z /= Void
		local
			x2, y2, div, tmp1, tmp2: MPFR_REF
			prec: NATURAL_32
		do
			prec := z.real.precision.max (z.imaginary.precision)
			x2 := z.real + z.real
			x2.set_precision (re_prec)
			x2.set_rounding_mode (re_rnd)
			y2 := z.imaginary + z.imaginary
			y2.set_precision (re_prec)
			y2.set_rounding_mode (re_rnd)
			create tmp1.make_precision_nan (re_prec, re_rnd)
			last_result := mpfr_cos (tmp1.item, x2.item, x2.rounding_mode)
			create tmp2.make_precision_nan (im_prec, im_rnd)
			last_result := mpfr_cosh (tmp2.item, y2.item, y2.rounding_mode)
			div := tmp1 / tmp2
			last_result := mpfr_sin (x2.item, x2.item, x2.rounding_mode)
			last_result := mpfr_sinh (y2.item, y2.item, y2.rounding_mode)
			create Result.make (x2 / div, y2 / div)
		end

feature {NONE} -- Implementation

	One: MPFR_COMPLEX_REF is
			-- One.
		once
			create Result
			Result := Result.one
		end

	times_i (v: MPFR_COMPLEX_REF): MPFR_COMPLEX_REF is
			-- Current multiplied by `i'.
		do
			create Result.make (v.real.zero, v.real + v.imaginary)
		end

	Two: MPFR_COMPLEX_REF is
			-- Two.
		once
			Result := One + One
		end

	Two_mpfr: MPFR_REF is
			-- Two.
		once
			create Result
			Result.set_from_integer_32 (1)
		end

	Ten: MPFR_COMPLEX_REF is
			-- Ten.
		once
			Result := Two + Two + Two + Two + Two
		end

end
