indexing
	description: "Mathematical operations on floating-point numbers."
	author: "Christopher John Saunders"
	date: "$Date: 2006-12-05 (Tues, 05 Dec 2006) $"
	revision: "$Revision: 2$"

class
	MPFR_MATH

inherit
	MPFR_FUNCTIONS
		export
			{NONE} all
		end

feature -- Access

	last_result: INTEGER_32
			-- Last result of a call to
			-- an MPFR function.

feature -- Constants

	log_of_2 (prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Logarithm of 2 with precision `prec'
			-- rounded in the direction `rnd'.
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_const_pi (Result.item, rnd)
		ensure
			result_not_void: Result /= Void
		end

	pi (prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Pi with precision `prec'
			-- rounded in the direction `rnd'.
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_const_pi (Result.item, rnd)
		ensure
			result_not_void: Result /= Void
		end

	euler (prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Euler's constant 0.577...
			-- with precision `prec'
			-- rounded in the direction `rnd'.
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_const_euler (Result.item, rnd)
		ensure
			result_not_void: Result /= Void
		end

	catalan (prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Catalan's constant 0.915...
			-- with precision `prec'
			-- rounded in the direction `rnd'.
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_const_catalan (Result.item, rnd)
		ensure
			result_not_void: Result /= Void
		end

feature -- Logarithm

	log (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Natural logarithm of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_log (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	log2 (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Base 2 logarithm of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_log2 (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	log10 (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Base 10 logarithm of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_log10 (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

feature -- Power

	exp (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Exponential of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_exp (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	exp2 (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- 2 ^ `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_exp2 (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	exp10 (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- 10 ^ `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_exp10 (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	pow (v, w: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Raise `v' to the power `w'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
			w_not_void: w /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_pow (Result.item, v.item, w.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

feature -- Root extraction

	sqrt (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Square root of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
			v_not_negative: not (v < v.zero)
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_sqrt (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	cbrt (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Cubic root of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_cbrt (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	root (v: MPFR_REF; n: NATURAL_32; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- `n'th root of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
			n_positive: n > 0
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_root (Result.item, v.item, n, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

feature -- Hyperbolic

	cosh (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Cosine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_cosh (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	coth (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Tangent of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_coth (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	csch (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Sine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_csch (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	sech (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Cosine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_sech (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	sinh (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Sine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_sinh (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	tanh (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Tangent of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_tanh (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

feature -- Inverse hyperbolic

	acosh (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Cosine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_acosh (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	asinh (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Sine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_asinh (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	atanh (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Tangent of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_atanh (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

feature -- Trigonometric

	cos (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Cosine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_cos (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	cot (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Tangent of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_cot (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	csc (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Sine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_csc (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	sec (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Cosine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_sec (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	sin (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Sine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_sin (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	sin_cos (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): TUPLE [MPFR_REF, MPFR_REF] is
			-- Sine and cosine of `v'.
			-- The first item in the result is the
			-- sine of `v' and the second is the
			-- cosine.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		local
			res1, res2: MPFR_REF
		do
			create res1.make_precision_nan (prec, rnd)
			create res2.make_precision_nan (prec, rnd)
			last_result := mpfr_sin_cos (res1.item, res2.item, v.item, rnd.item)
			create Result
			Result[1] := res1
			Result[2] := res2
		ensure
			result_not_void: Result /= Void
		end

	tan (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Tangent of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_tan (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

feature -- Inverse trigonometric

	acos (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Arc cosine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_acos (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	asin (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Arc sine of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_asin (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	atan (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Arc tangent of `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_atan (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	atan2 (y, x: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Arc tangent of `y' / `x'.
			-- Using the same rounding mode of `y'.
		require
			y_not_void: y /= Void
			x_not_void: x /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_atan2 (Result.item, y.item, x.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

feature -- Miscellaneous

	gamma (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Gamma function on `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_gamma (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	zeta (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Riemann Zeta function on `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_zeta (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	erf (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Error function on `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_erf (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	erfc (v: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Complimentary error function on `v'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_erfc (Result.item, v.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	agm (v, w: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Arithmetic-geometric mean of `v' and `w'.
			-- Using the same rounding mode of `v'.
		require
			v_not_void: v /= Void
			w_not_void: w /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_agm (Result.item, v.item, w.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

	hypot (x, y: MPFR_REF; prec: NATURAL_32; rnd: MPFR_RND_T): MPFR_REF is
			-- Euclidean norm of `x' and `y'.
			-- Using the same rounding mode of `x'.
		require
			x_not_void: x /= Void
			y_not_void: y /= Void
		do
			create Result.make_precision_nan (prec, rnd)
			last_result := mpfr_hypot (Result.item, x.item, y.item, rnd.item)
		ensure
			result_not_void: Result /= Void
		end

end
