indexing
	description: "Objects that ..."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	MP_COMPLEX [G -> MP_NUMBER create default_create end]

inherit
	DEBUG_OUTPUT
		rename
			debug_output as out
		redefine
			copy,
			default_create,
			is_equal,
			out
		end

	NUMERIC
		redefine
			copy,
			default_create,
			is_equal,
			out
		end

create
	default_create,
	make,
	make_real

feature {NONE} -- Initialization

	default_create is
			-- Initialize `Current'.
		do
			create real
			create imaginary
		end

	make (re, im: like real) is
			-- Initialize `Current'.
			-- Set `real' to `re' and `imaginary' to `im'.
		do
			real := re
			imaginary := im
		ensure
			real_set: real = re
			imaginary_set: imaginary = im
		end

	make_real (re: like real) is
			-- Initialize `Current' as a real number.
		do
			real := re
			create imaginary
		ensure
			real_set: real = re
			imaginary_zero: imaginary.is_equal (imaginary.zero)
		end

feature -- Access

	imaginary: like real
			-- Imaginary part.

	one: like Current is
			-- Neutral element for "*" and "/"
		do
			create Result
			Result.set (real.one, imaginary.zero)
		end

	quadrant: INTEGER_32 is
		local
			z: like real
		do
			z := real.zero
			if ((real >= z) and then (imaginary >= z)) then
				Result := 1
			elseif ((real < z) and then (imaginary >= z)) then
				Result := 2
			elseif ((real <= z) and then (imaginary < z)) then
				Result := 3
			elseif ((real >= z) and then (imaginary < z)) then
				Result := 4
			end
		end

	real: G
			-- Real part.

	zero: like Current is
			-- Neutral element for "+" and "-"
		do
			create Result
		end

feature -- Status report

	divisible (other: like Current): BOOLEAN is
			-- May current object be divided by `other'?
		do
			Result := not other.is_equal (zero)
		end

	exponentiable (other: NUMERIC): BOOLEAN is
			-- May current object be elevated to the power `other'?
			-- Default: False
		do
			Result := False
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		do
			Result := real.is_equal (other.real) and then imaginary.is_equal (other.imaginary)
		end

feature -- Element change

	set (re, im: like real) is
			-- Set `real' to `re' and `imaginary' to `im'.
		do
			real := re
			imaginary := im
		ensure
			real_set: real = re
			imaginary_set: imaginary = im
		end

	set_imaginary (im: like real) is
			-- Set `imaginary' to `im'.
		do
			imaginary := im
		ensure
			imaginary_set: imaginary = im
		end

	set_real (re: like real) is
			-- Set `real' to `re'.
		do
			real := re
		ensure
			real_set: real = re
		end

feature -- Conversion

	as_real: like real is
			-- Convert to a real number.
		do
			Result := real.twin
		ensure
			result_not_void: Result /= Void
		end

	to_real: like real is
			-- Convert to a real number.
		require
			imaginary_zero: imaginary.is_equal (imaginary.zero)
		do
			Result := real.twin
		ensure
			result_not_void: Result /= Void
		end

feature -- Duplication

	copy (other: like Current) is
			-- Update current object using fields of object attached
			-- to `other', so as to yield equal objects.
		do
			real.copy (other.real)
			imaginary.copy (other.imaginary)
		end

feature -- Basic operations

	infix "+" (other: like Current): like Current is
			-- Sum with `other' (commutative).
		do
			create Result
			Result.set (real + other.real, imaginary + other.imaginary)
		end

	infix "-" (other: like Current): like Current is
			-- Result of subtracting `other'
		do
			create Result
			Result.set (real - other.real, imaginary - other.imaginary)
		end

	infix "*" (other: like Current): like Current is
			-- Product by `other'
		do
			create Result
			Result.set (real * other.real - imaginary * other.imaginary,
			            real * other.imaginary + imaginary * other.real)
		end

	infix "/" (other: like Current): like Current is
			-- Division by `other'
		local
			div: like real
		do
			div := other.real * other.real + other.imaginary * other.imaginary
			create Result
			Result.set ((real * other.real + imaginary * other.imaginary) / div,
			            (imaginary * other.real - real * other.imaginary) / div)
		end

	prefix "+": like Current is
			-- Unary plus
		do
			create Result
			Result.set (+real, +imaginary)
		end

	prefix "-": like Current is
			-- Unary minus
		do
			create Result
			Result.set (-real, -imaginary)
		end

	squared: like Current is
			-- `Current' raised to the power 2.
		do
			create Result
			Result.set (real * real - imaginary * imaginary, (real.one + real.one) * real * imaginary)
		end

feature -- Output

	out: STRING is
			-- New string containing terse printable representation
			-- of current object
		do
			if real.is_equal (real.zero) then
				if imaginary.is_equal (imaginary.zero) then
					Result := imaginary.out
				else
					Result := imaginary.out + "i"
				end
			else
				Result := real.out
				if not imaginary.is_equal (imaginary.zero) then
					if imaginary > imaginary.zero then
						Result.append_string (" + " + imaginary.out + "i")
					else
						Result.append_string (" - " + (-imaginary).out + "i")
					end
				end
			end
		end

invariant
	real_not_void: real /= Void
	imaginary_not_void: imaginary /= Void

end
